<?php

namespace App\Jobs;

use App\Models\EmailService;
use App\Services\EmailSendingService;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Config;

class ProcessEmailBatch implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected EmailService $emailService;
    public $timeout = 3600;    // 1 hora
    public $tries = 1;         // Não tenta novamente
    public $maxExceptions = 1; // Falha após 1 exceção

    public function __construct(EmailService $emailService)
    {
        $this->emailService = $emailService;
        $this->onQueue('emails');
    }

    public function handle()
    {
        // Usa transaction para garantir atomicidade do lock
        $locked = DB::transaction(function () {
            $service = EmailService::lockForUpdate()->find($this->emailService->id);
            Log::info("Service {$this->emailService->name} is processing: {$service->is_processing} ");
            if ($service->is_processing) {
                Log::info("Service {$this->emailService->name} is not processing: {$service->is_processing} ");
                return false;
            }
            $service->update(['is_processing' => true]);
            Log::info("Service {$this->emailService->name} is processing: {$service->is_processing} ");
            return true;
        });

        if (!$locked) {
            Log::info("Service {$this->emailService->name} is already processing emails. Skipping. processing emails but not locked");
            return;
        }

        try {
            Log::info("Starting ProcessEmailBatch for service: {$this->emailService->name}");

            // Seleciona o próximo lote de emails pendentes
            $pendingEmails = $this->emailService->emailQueue()
                ->where('status', 'pending')
                ->orderBy('created_at')
                ->take($this->emailService->batch_size)
                ->get();

            Log::info("Found {$pendingEmails->count()} emails to process");

            foreach ($pendingEmails as $email) {
                try {
                    $email->update(['status' => 'processing']);
                    
                    // Usar o serviço de envio de email
                    $emailSender = new EmailSendingService();
                    $success = $emailSender->send($email, $this->emailService);

                    echo("\n:: ProcessEmailBatch: " . $success . " - " . $email->id . "\n");

                    if ($success) {
                        $email->update([
                            'status' => 'sent',
                            'sent_at' => now()
                        ]);
                    } else {
                        throw new \Exception("Failed to send email");
                    }

                } catch (\Exception $e) {
                    echo("\n:: ProcessEmailBatch - ERROR: \n". $e->getMessage());
                    Log::error("Error processing email #{$email->id}: " . $e->getMessage());
                    $email->update([
                        'status' => 'failed',
                        'error' => $e->getMessage()
                    ]);
                }
            }

            // Se ainda houver emails pendentes, agenda o próximo lote
            $remainingCount = $this->emailService->emailQueue()
                ->where('status', 'pending')
                ->count();

            Log::info("Remaining pending emails: {$remainingCount}");

            if ($remainingCount > 0) {
                Log::info("Scheduling next batch for service: {$this->emailService->name}");
                static::dispatch($this->emailService)
                    ->delay(now()->addSeconds($this->emailService->batch_interval));
            }

        } catch (\Exception $e) {
            Log::error("ProcessEmailBatch error: " . $e->getMessage());
            throw $e;
        } finally {
            // Sempre reseta o flag, mesmo em caso de erro
            DB::transaction(function () {
                $service = EmailService::lockForUpdate()->find($this->emailService->id);
                $service->update(['is_processing' => false]);
            });
        }
    }
} 